@@ -64,11 +64,16 @@ val boxedint32_header : nativeint
 val boxedint64_header : nativeint
 val boxedintnat_header : nativeint
 
+(** Closure info for a closure of given arity and distance to environment *)
+val closure_info : arity:int -> startenv:int -> nativeint
+
 (** Wrappers *)
 val alloc_float_header : Debuginfo.t -> expression
 val alloc_floatarray_header : int -> Debuginfo.t -> expression
 val alloc_closure_header : int -> Debuginfo.t -> expression
 val alloc_infix_header : int -> Debuginfo.t -> expression
+val alloc_closure_info :
+      arity:int -> startenv:int -> Debuginfo.t -> expression
 val alloc_boxedint32_header : Debuginfo.t -> expression
 val alloc_boxedint64_header : Debuginfo.t -> expression
 val alloc_boxedintnat_header : Debuginfo.t -> expression
@@ -153,6 +158,10 @@ val mk_if_then_else :
 (** Boolean negation *)
 val mk_not : Debuginfo.t -> expression -> expression
 
+(** Integer and float comparison that returns int not bool *)
+val mk_compare_ints : Debuginfo.t -> expression -> expression -> expression
+val mk_compare_floats : Debuginfo.t -> expression -> expression -> expression
+
 (** Loop construction (while true do expr done).
     Used to be represented as Cloop. *)
 val create_loop : expression -> Debuginfo.t -> expression
@@ -180,6 +189,12 @@ val remove_unit : expression -> expression
 
 (** Blocks *)
 
+(** Non-atomic load of a mutable field *)
+val mk_load_mut : memory_chunk -> operation
+
+(** Atomic load. All atomic fields are mutable. *)
+val mk_load_atomic : memory_chunk -> operation
+
 (** [field_address ptr n dbg] returns an expression for the address of the
     [n]th field of the block pointed to by [ptr] *)
 val field_address : expression -> int -> Debuginfo.t -> expression
@@ -309,6 +324,9 @@ val check_bound :
   expression -> expression -> expression ->
   expression
 
+(** Sys.opaque_identity *)
+val opaque : expression -> Debuginfo.t -> expression
+
 (** Generic application functions *)
 
 (** Get the symbol for the generic application with [n] arguments, and
@@ -526,12 +544,12 @@ val make_switch :
 
 (** [transl_int_switch loc arg low high cases default] *)
 val transl_int_switch :
-  Location.t -> expression -> int -> int ->
+  Debuginfo.t -> expression -> int -> int ->
   (int * expression) list -> expression -> expression
 
 (** [transl_switch_clambda loc arg index cases] *)
 val transl_switch_clambda :
-  Location.t -> expression -> int array -> expression array -> expression
+  Debuginfo.t -> expression -> int array -> expression array -> expression
 
 (** [strmatch_compile dbg arg default cases] *)
 val strmatch_compile :
@@ -591,10 +609,6 @@ val globals_map:
     from the given compilation units *)
 val frame_table: string list -> phrase
 
-(** Generate the caml_spacetime_shapes table, referencing the spacetime shapes
-    from the given compilation units *)
-val spacetime_shapes: string list -> phrase
-
 (** Generate the tables for data and code positions respectively of the given
     compilation units *)
 val data_segment_table: string list -> phrase
